#!/usr/bin/env python3

import helpers as h

import re
from ctypes import c_void_p

# P-App-Param
class appParam:

	def __init__(self, p_app_param):
		# generic
		self.sound_set_id = h.getAppParams(p_app_param, 'sound_set_id')
		self.contract_sound_set_id = h.getAppParams(p_app_param, 'contract_sound_set_id')
		self.audio_id = h.getAppParams(p_app_param, 'audio_id')
		self.play_looped = h.getAppParams(p_app_param, 'play_looped')
		self.audio_name = h.getAppParams(p_app_param, 'audio_name')
		self.fr_code = h.getAppParams(p_app_param, 'fr_code')
		self.fr_reason = h.getAppParams(p_app_param, 'fr_reason')
		self.calltimer = h.getAppParams(p_app_param, 'calltimer')
		self.uuid = h.getAppParams(p_app_param, 'uuid')
		self.first_caller_uri = h.getAppParams(p_app_param, 'first_caller_uri')
		self.first_caller_ext = h.getAppParams(p_app_param, 'first_caller_ext')
		self.first_caller_pbx_account_id = h.getAppParams(p_app_param, 'first_caller_pbx_account_id')
		self.first_caller_clir = h.getAppParams(p_app_param, 'first_caller_clir')
		self.phone_hdrs = h.parse_phone_headers(p_app_param, 'phone_hdrs')
		self.phone_uri = h.getAppParams(p_app_param, 'phone_uri')

		# queues
		self.max_queue_length = h.getAppParams(p_app_param, 'max_queue_length')
		self.callee_wrap_up_time = h.getAppParams(p_app_param, 'callee_wrap_up_time')

		# conference
		self.room = h.getAppParams(p_app_param, 'room')
		self.roompin = h.getAppParams(p_app_param, 'roompin')

		# auto-attendant
		self.aa_account_id = h.getAppParams(p_app_param, 'aa_account_id')
		self.aa_account_uri = h.getAppParams(p_app_param, 'aa_account_uri')

		# call through / calling card
		self.srcnumber = h.getAppParams(p_app_param, 'srcnumber')
		self.outgoing_cli = h.getAppParams(p_app_param, 'outgoing_cli')
		self.currency = h.getAppParams(p_app_param, 'currency')

# global session parameters
class sessionParam:

	def __init__(self, session):
		self.callee_uuid = session.getVariable('sip_h_P-Callee-UUID')
		self.caller_uuid = session.getVariable('sip_h_P-Caller-UUID')
		self.from_tag = session.getVariable('sip_from_tag')
		self.to_tag = session.getVariable('sip_to_tag')
		self.from_uri = session.getVariable('sip_from_uri')
		self.to_uri = session.getVariable('sip_to_uri')
		self.from_user = session.getVariable('sip_from_user')
		self.to_user = session.getVariable('sip_to_user')
		self.from_host = session.getVariable('sip_from_host')
		self.to_host = session.getVariable('sip_to_host')
		self.callid = session.getVariable('sip_call_id')

		# hunt groups
		self.hg_members = session.getVariable('sip_h_P-Huntgroup-Members')
		self.hg_policy = session.getVariable('sip_h_P-Hunting-Policy')
		self.hg_timeout = session.getVariable('sip_h_P-Hunting-Timeout')
		self.hg_cancel_mode = session.getVariable('sip_h_P-Hunting-Cancel-Mode')

#
# calling card / call through related
#
class reMatchApp(str):
	def __eq__(self, pattern):
		return bool(re.search(pattern, self))

class callThrough:

	def __init__(self, _source_uuid, _auth_username, _auth_password, _sw_domain):
		self.source_uuid = _source_uuid
		self.auth_username = _auth_username
		self.auth_password = _auth_password
		self.sw_domain = _sw_domain

class callingCard:

	def __init__(self, _source_uuid, _auth_username, _auth_password, _sw_domain):
		self.source_uuid = _source_uuid
		self.auth_username = _auth_username
		self.auth_password = _auth_password
		self.sw_domain = _sw_domain

class swrProfile:

	def __init__(self, _contract_id, _profile_id, _prepaid, _interval_charge, _interval_free_time,
						_interval_free_cash, _interval_unit, _interval_count, _prepaid_library, _ignore_domain):
		self.contract_id = _contract_id
		self.profile_id = _profile_id
		self.prepaid = _prepaid
		self.interval_charge = _interval_charge
		self.interval_free_time = _interval_free_time
		self.interval_free_cash = _interval_free_cash
		self.interval_unit = _interval_unit
		self.interval_count = _interval_count

		# for extended swr profiles, TODO: should we add support? (see `_swr_profile_is_extended()` in libswrate)
		self.prepaid_library = _prepaid_library
		self.ignore_domain = _ignore_domain

class ccBalance:

	def __init__(self, _balance, _cents, _free_seconds, _units):
		self.balance = _balance
		self.cents = _cents
		self.free_seconds = _free_seconds
		self.units = _units

class SWRATE(c_void_p):
    # subclassing c_void_p creates an opaque pointer type that is distinct
    # from c_void_p, and can only be instantiated as a pointer
    pass
