#!/usr/bin/env python3

import yaml
import MySQLdb
import time
from pathlib import Path

import general_definitions as global_defs
from data_types import callThrough, callingCard, swrProfile

config = '/etc/freeswitch/dialplan/logic/db_definitions.yml'
with open(config, 'r') as file:
    data_loaded = yaml.safe_load(file)

# generally used mysql credentials
dbuser = data_loaded['config']['provisioning']['user']
dbpassword = data_loaded['config']['provisioning']['password']
dbhost = data_loaded['config']['provisioning']['host']
dbport = data_loaded['config']['provisioning']['port']
dbname = data_loaded['config']['provisioning']['database']

# CC related credentials
cc_dbuser = data_loaded['config']['cc']['user']
cc_dbpassword = data_loaded['config']['cc']['password']
cc_dbhost = data_loaded['config']['cc']['host']
cc_dbport = data_loaded['config']['cc']['port']
cc_dbname_billing = data_loaded['config']['cc']['database_billing']
cc_dbname_accounting = data_loaded['config']['cc']['database_accounting']

# audio files fetcher
def DBfetchAudioFile(session, sound_set_id: str, audio_name: str):

	file_dir = str(global_defs.audio_cache_directory + '/' + sound_set_id)
	file_path = str(file_dir + '/' + audio_name + global_defs.default_audio_extension)

	# use pathlib for required checks
	file = Path(file_path)
	p = Path(file_dir)

	# check if the directory for this sound_set_id has been created
	if (not p.exists()):
		p.mkdir(parents=True, exist_ok=True)

	# check if a soundfile already file exists
	if (file.exists()):
		session.execute('log', "File: '" + str(file_path) + "' already exists, no need to store again")
		return True

	try:
		conn = MySQLdb.connect(host=dbhost, port=int(dbport), user=dbuser, password=dbpassword, database=dbname)
		cursor = conn.cursor()
		query = f"SELECT data FROM v_sound_set_files WHERE set_id='{sound_set_id}' AND handle_name='{audio_name}'"

		cursor.execute(query)
		result = cursor.fetchone()

		cursor.close()
		conn.close()

		if result and result[0]:
			file_data = result[0]
			with open(file_path, 'wb') as file:
				file.write(file_data)
			session.execute('log', "File saved to: '" + str(file_path) + "'")
			return True
		else:
			session.execute('log', "Audio file not found in the database: '" + str(audio_name) + "'")
			return False

	except MySQLdb.Error as err:
		session.execute('log', "Error: '" + str(err) + "'")

# AA destinations fetcher
def DBfetchAADestination(session, uuid: str, choice: str, default: bool):
	try:
		conn = MySQLdb.connect(host=dbhost, port=int(dbport), user=dbuser, password=dbpassword, database=dbname)
		cursor = conn.cursor()
		query = None
		if (default):
			query = f"SELECT destination FROM voip_pbx_autoattendants WHERE uuid='{uuid}' AND choice='default'"
		else:
			query = f"SELECT destination FROM voip_pbx_autoattendants WHERE uuid='{uuid}' AND choice='{choice}'"

		cursor.execute(query)
		result = cursor.fetchone()

		cursor.close()
		conn.close()

		if result and result[0]:
			return str(result[0])
		return None

	except MySQLdb.Error as err:
		session.execute('log', "Error: '" + str(err) + "'")

# fetch parameters for call-through
def DBfetchCallThroughParameters(session, uuid: str, srcnumber: str):
	try:
		conn = MySQLdb.connect(host=dbhost, port=int(dbport), user=dbuser, password=dbpassword, database=dbname)
		cursor = conn.cursor()
		query = (f"SELECT m.source_uuid AS source_uuid, s.username AS auth_username, s.password AS auth_password,"
				f"d.domain as sw_domain FROM provisioning.voip_cc_mappings m,provisioning.voip_subscribers s, "
				f"provisioning.voip_domains d WHERE m.uuid='{uuid}' AND m.auth_key='{srcnumber}' AND m.source_uuid=s.uuid AND s.domain_id=d.id")

		cursor.execute(query)
		result = cursor.fetchone()

		cursor.close()
		conn.close()

		if (result and result[0] and result[1] and result[2] and result[3]):
			return callThrough(result[0], result[1], result[2], result[3])

		# authentication is missing
		session.execute('log', ("Error: Some important parameters missing "
								f"(source_uuid: '{result[0]}', auth_username: '{result[1]}', "
								f"auth_password: '{result[2]}' , sw_domain: '{result[3]}' )"))
		return None

	except MySQLdb.Error as err:
		session.execute('log', "Error: '" + str(err) + "'")

# fetch parameters for calling-card
def DBfetchCallingCardParameters(session, uuid: str, pin: str):
	try:
		conn = MySQLdb.connect(host=dbhost, port=int(dbport), user=dbuser, password=dbpassword, database=dbname)
		cursor = conn.cursor()
		query = (f"SELECT m.source_uuid AS source_uuid, s.username AS auth_username, s.password AS auth_password,"
				f"d.domain as sw_domain FROM provisioning.voip_cc_mappings m,provisioning.voip_subscribers s, "
				f"provisioning.voip_domains d WHERE m.uuid='{uuid}' AND m.auth_key='{pin}' AND m.source_uuid=s.uuid AND s.domain_id=d.id")

		cursor.execute(query)
		result = cursor.fetchone()

		cursor.close()
		conn.close()

		if (result and result[0] and result[1] and result[2] and result[3]):
			return callingCard(result[0], result[1], result[2], result[3])

		# authentication is missing
		session.execute('log', "Error: Some important parameters missing (DBfetchCallingCardParameters).")
		return None

	except MySQLdb.Error as err:
		session.execute('log', "Error: '" + str(err) + "'")

# fetch accound id for calling-card
def DBgetVoipAccountBySubscriber(session, source_uuid: str):
	cur_time = str(time.time())

	try:
		conn = MySQLdb.connect(host=dbhost, port=int(dbport), user=dbuser, password=dbpassword, database=dbname)
		cursor = conn.cursor()
		query = (f"SELECT s.contract_id, bp.id, bp.prepaid, bp.interval_charge, bp.interval_free_time, bp.interval_free_cash, "
				f"bp.interval_unit, bp.interval_count, bp.prepaid_library, bp.ignore_domain "
				f"FROM billing.voip_subscribers s "
				f"JOIN (SELECT p.* FROM billing.billing_profiles p WHERE p.id = billing.get_billing_profile_by_uuid('{source_uuid}','{cur_time}')) bp "
				f"WHERE s.uuid = '{source_uuid}'")

		cursor.execute(query)
		result = cursor.fetchone()

		cursor.close()
		conn.close()

		# presence of at least contract_id is critical
		if (result and result[0]):
			return swrProfile(result[0], result[1], result[2],
								result[3], result[4], result[5],
								result[6], result[7], result[8],
								result[9])
		else:
			return None

	except MySQLdb.Error as err:
		session.execute('log', "Error: '" + str(err) + "'")
