#!/usr/bin/env python3

# libraries
import freeswitch
import sys
from importlib import import_module

# other local implementations
import announcement as an
import callqueues as cq
import conference as cnf
import auto_attendant as aa
import hunt_groups as hg

# helpers
import helpers as h
import db_helpers as dbh

# data types
from data_types import appParam
from data_types import sessionParam
from data_types import reMatchApp

# hangup callback handler function that will get called when the call terminates
def hangup_hook(session, what):
		session.execute('log', "hangup hook for '" + str(what) + "'")
		return

# default gateway into applications processing
def handler(session, args):
		callid = str(session.getVariable('sip_call_id'))

		# fetch app params required for processing
		p_app_name = str(session.getVariable('sip_h_P-App-Name'))
		p_app_param = str(session.getVariable('sip_h_P-App-Param'))
		app_params = appParam(p_app_param)
		sdp_body = str(session.getVariable('switch_r_sdp'))

		# fetch session params required for processing
		session_params = sessionParam(session)

		destination_app = str(session.getVariable('destination_number'))
		if (session_params.hg_members):	# HGs are detected using `P-Huntgroup-Members`
			destination_app = 'hunt-groups'

		session.execute('log', "Call to application: '" + destination_app + "', callid: '" + callid + "'")
		session.execute('log', "P-App-Name: '" + p_app_name + "' / P-App-Param: '" + p_app_param + "'")

		# go and find application
		session.setHangupHook(hangup_hook)
		if ('pro' in args):
			module_name = import_module('select_applications')
			meth = getattr(module_name, 'get_application')
			meth(session, destination_app, app_params, session_params, sdp_body, callid)
		else:
			module_name = import_module('select_applications_ce')
			meth = getattr(module_name, 'get_application')
			meth(session, destination_app, app_params, session_params, sdp_body, callid)

		# return to the XML dialplan
		return
