#!/usr/bin/env python3

import general_definitions as global_defs
import math

def getAppParams(source_str: str, variable_name: str):
	variables = source_str.split(';')
	for variable in variables:
		if not variable or '=' not in variable:
			continue
		name, value = variable.split('=', 1)  # Split only at the first '='
		if name.strip() == variable_name:
			return value.strip()
	return None

def getNameFromUri(sip_uri: str):
	if 'sip:' in sip_uri:
		user_part = sip_uri.split('@')[0].split('sip:')[-1]
		return user_part
	return None

def parse_phone_headers(source_str: str, variable_name: str):
	variables = source_str.split(';')
	for variable in variables:
		if not variable or '=' not in variable:
			continue
		name, value = variable.split('=', 1)  # Split only at the first '='
		if name.strip() == variable_name:
			lines = value.replace("\r\n", "\\r\\n")
			lines = lines.split('\\r\\n')
			parsed_values = []
			for line in lines:
				if not line:
					continue
				k, v = line.split(': ', 1)
				parsed_values.append((k, v))
			return parsed_values
	return None

# extension must be given in format `.wav` (for example), but can be empty
def formatAudioPath(sound_set_id: str, audio_file: str, extension: str):
	extension = extension if (extension != None) else str(global_defs.default_audio_extension)
	# set proper audio path
	return str(global_defs.audio_cache_directory + '/' + sound_set_id + '/' + audio_file + extension)

# Properly format digits to be played.
# returns an array of digits (playback files) as a list.
#
# cnt - billing.units
# basedir - audio files base directory, can be empty
# suffix - can be empty
# right or left - left for German language, right for the rest
def utilsGetCountFiles(cnt: int, basedir: str, suffix: str, right: bool):
    res = []
    number = cnt
    n = int(math.log10(number)) + 1
    max_digits = 9

    full_number = str(number)
    remainder = ''  # what remains after 9 digits, if remains
    remainder_exists = False

	# german-based languages start adding single digits (with and)
	# after 20
    if (cnt <= 20):
        res.append(basedir + str(cnt) + suffix)
        return res

	# extract the reminder after 9 digits
    if (n > max_digits):
        remainder_exists = True
        remainder = full_number[max_digits:]
        cnt = int(remainder)

	# add digits to be played
    for i in range(n, 0, -1):
        num = int((number % (10 ** i)) // (10 ** (i - 1)))
        if (n - i) < max_digits:
            res.append(basedir + str(num) + suffix)  # only push until max amount here

	# if no reminder, just finish now
    if (not remainder_exists):
        return res

	# add reminder
    if (cnt <= 20 or cnt % 10 == 0):
        res.append(basedir + str(cnt) + suffix)
        return res

    quot, rem = divmod(cnt, 10)
    if right:
        # language has single digits before 10s
        res.append(basedir + str(quot * 10) + suffix)
        res.append(basedir + str(rem) + "-and" + suffix)
    else:
        # language has single digits before 10s
        res.append(basedir + str(rem) + "-and" + suffix)
        res.append(basedir + str(quot * 10) + suffix)

    return res
